/**
 * The applet which takes the logical expression from the text window
 * and displays the truth table step by step
 * @ author Jobby Jacob
 */

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.math.*;
import java.io.*;
import java.util.*;


public class truthTableApplet extends Applet {
    
    protected truthTable t;

    private boolean clear=true;
    private boolean next=false;
    private boolean error=false;
    private boolean zeroOne = false;
    //default is T/F
    private int i;
   
    /*The push down buttons*/
    
    protected Button submitButton;
    protected Button nextButton;
    protected Button clearButton;
    protected Button switchButton;
    
    /* The text window */
    protected TextField tf;
   
    private String input,exp;
    
    
    public truthTableApplet() {
    }


    
    public void init() 
    {
        Font label=new Font("Monospaced", Font.BOLD, 14);
        setLayout(new FlowLayout());
        Label info1 = new Label("Use '~' for NEGATE, '*' or '/\\' for AND and '+' or '\\/' for OR.");
        Label info2 = new Label("Use => for implication and <=> for equivalence.");
        Label info3=new Label("You can use either lower case or upper case letters.");
        Label info4=new Label("Press Switch button to switch between 0/1 and T/F. Default is T/F.");
        
        info1.setFont(label);
        info2.setFont(label);
        info3.setFont(label);
        info4.setFont(label);
        
        info1.setForeground(Color.red);
        info2.setForeground(Color.red);
        info3.setForeground(Color.red);
        info4.setForeground(Color.red);
        
        tf = new TextField("",50);
        switchButton = new Button("Switch");
        submitButton = new Button("Submit");
        nextButton = new Button("Next");
        clearButton=new Button("Clear");
        
        submitButton.setBackground(Color.cyan);
        submitButton.setForeground(Color.black);
        add(info1);
        add(info2);
        add(info3);
        add(info4);
        
        add(tf);
        add(switchButton);
        add(submitButton);
        add(nextButton);
        add(clearButton);
    }


    
    public void paint(Graphics g) 
    {
        if(!clear)
        {
            g.drawString(exp, 5, 140); /*Printing out the entered expression */
            if(error){
                g.setColor(Color.red);
                g.drawString("Invalid expression 0",5,160); /* Printing out the error message */
                g.setColor(Color.black);
            }else{
                /* Printing out the first row of the table which are characters */
                g.setColor(Color.black); 
                String s="";
                int c=0,n=0,l=0;
                for(int h=0;h<input.length();h++){
                    if(input.charAt(h)=='('){
                        s=s+input.charAt(h);
                    }else if(input.charAt(h)==')'){
                      n++;//indicating end of string?
                      s=s+input.charAt(h);
                      g.drawString(s,4*(n+l)+5+(c-1)*50, 160);
                      s="";
                            
                        
                    }else{
                        s=s+input.charAt(h);
                        g.drawString(s,5+c*50,160);
                        c++;
                        n=0;
                        l=s.length();
                        s="";
                    }
                }
                g.setColor(Color.black);
                /* Printing out the columns in the order of operations performed */
                for(int j=0;j<i;j++){
                    if(Character.isLetter(t.result[j].token.charAt(j))){
                        g.setColor(Color.blue);
                    }else{
                        g.setColor(Color.black);
                    }
                    if(j==t.count-1){
                        g.setColor(Color.yellow);
                        g.fillRect(t.result[j].column*50, 170, 20, 15*(int)Math.pow(2,t.charCount));
                        g.setColor(Color.red);
                    }
                    for(int k=0;k<(int)Math.pow(2,t.charCount);k++){
                        if(t.table[k][t.result[j].column]==1)
                        {
                            if(zeroOne)
                            {
                            	g.drawString("1",5+t.result[j].column*50,180+(15*k));
                            }
                            else
                            {
                            	g.drawString("T",5+t.result[j].column*50,180+(15*k));
                            }
                        }
                        else
                        {
                            if (zeroOne)
                            {
                            	g.drawString("0",5+t.result[j].column*50,180+(15*k));
                            }
                            else
                            {
                            	g.drawString("F",5+t.result[j].column*50,180+(15*k));
                            }
                        }
                    }
                }
            }
        }
    }
    
    
    public boolean action(Event e, Object arg) 
    {
        if (e.target == switchButton)
        {
        	if (!zeroOne)
        	{
        	  zeroOne = true;
          	}
        	else
        	{
        	  zeroOne = false;
        	}
        	error = false;
        	next=true; //there is a next step after switch is pressed
        	clear = false;
        }
        else if (e.target == submitButton)
        {
            input=tf.getText();
            input = input.trim();
            
            if(input.length()>0)
            {
                i=0;
                exp="The entered expression is  ";
                exp=exp.concat(input);
                                
                //As soon as submit button is pressed, print the entered expression.
                next = true; //there is a next step after submit is pressed
                clear=false;
                error=false;
                
                //this portion gets the input string ready for building. 
                //Take each element as a token
                StringBuffer tempo = new StringBuffer();
    	    	
    	        for (int i=0;i<input.length();i++)
    	        {
    	        	char c1= input.charAt(i);
    	        	char c2 = input.charAt(i+1);
    	        	char c3 = input.charAt(i+2);
                    	       
    	        	if (Character.isLetter(c1))
    	        	{
    	        		tempo.append(c1);
    	        		tempo.append(" "); //append a whitespace after a character 
    	        	}
    	        	else if(c1=='*'||c1=='+'||c1=='('||c1==')')//all one-character tokens
    	        	{
    	        		tempo.append(c1);
    	        		tempo.append(" ");//append a whiteSpace after a one-character operator
    	        	}
    	        	else if(c1=='=')
    	        	{
    	        		if (c2=='>')
    	        		{
    	        			tempo.append(c1);
    	        			tempo.append(c2);
    	        			tempo.append(" ");
    	        		}//else ignore the = - it makes no sense
    	        	}
    	        	else if(c1=='/')
    	        	{
    	        		if (c2=='\\')
    	        		{
    	        			tempo.append(c1);
    	        			tempo.append(c2);
    	        			tempo.append(" ");
    	        		}
    	        	}
    	        	else if(c1=='\\')
    	        	{
    	        		if (c2=='/')
    	        		{
    	        			tempo.append(c1);
    	        			tempo.append(c2);
    	        			tempo.append(" ");
    	        		}
    	        	}
    	        	else if(c1=='<')
    	        	{
    	        		tempo.append(c1);
    	        	}
    	         }
    	    	
    	    	input = tempo.toString(); //this string has a space after each significant token.
    	    	//any letter other than those expected have been left out of the string
    	    	input.trim();
    	    	String list = " ";
    	    	StringTokenizer tokenString = new StringTokenizer(input, list);
    	    	                
                this.t = new truthTable(tokenString); //Creating an instance of truthtable
                
                
                // this portion gets the input string devoid of spaces for validity check
                StringBuffer temp=new StringBuffer();
                //to remove all whiteSpace
                for (int i=0;i<input.length();i++)
                {
                	char c = input.charAt(i);
                	System.out.println(c);
                	if (!Character.isWhitespace(c))
                	{
                		System.out.println(c);
                		temp.append(c);
                	}
                	
                }
                input = temp.toString();//this string is devoid of spaces        
                try
                {
                	t.validityCheck(input); //Checking for the validity of expression
                }
                catch(IOException er)
                {
                    error=true;
                }
                finally
                {
                    if(!error)
                    {
                        t.buildTable(tokenString);
                        t.treeBuild(t.result,t.count);
                        t.evalTree(t.result[t.count-1]);
                    }
               }
            }
        }
      else if(e.target==clearButton)
      {
            tf.setText("");  //Clearing of everything
            clear=true;
            next = false;// there is no next step after clear is pressed?
            error=false;
            i=0;
        }
      else if(e.target==nextButton)
      {
            if(next&&!error)
            {
                if(i<t.count)
                {
                    i++;
                }
            } 
        }
        repaint();
        return true;
    }


    public static void main(String args[])
    {
        MyWindow w=new MyWindow(450, 450, 50,50);
        Applet a = new truthTableApplet();
        w.add(a);
        a.init();
        w.showWindow();
    }
}